﻿local MAJOR_VERSION = "FuBarPlugin-2.0"
local MINOR_VERSION = "$Revision: 999999 $"

local broker = LibStub("LibDataBroker-1.1")
local AceOO = AceLibrary("AceOO-2.0")

local FuBarPlugin = AceOO.Mixin {
	"SetIcon",
	"SetText",
	"GetAceOptionsDataTable",
	"UpdateDisplay",
	"UpdateText",
	"GetTitle",
}

local brokerPlugins = {}
local folderNames = {}
local tablets = {}

function FuBarPlugin:OnEmbedInitialize(addon, name)
	local brokerPlugin = broker:NewDataObject(name)
	brokerPlugins[addon] = brokerPlugin
		
	addon:SetText(addon:GetTitle())
	addon:SetIcon(addon.icon or addon.hasIcon)
	
	brokerPlugin.OnClick = function(frame, button)
		if button == "RightButton" then
			FuBarPlugin:OpenMenu(frame, addon)
		end
	end
	
	brokerPlugin.OnEnter = function(frame)
		if not addon.blizzardTooltip and not tablets[frame] then
			FuBarPlugin:TabletRegister(frame, addon)
			tablets[frame] = true 
		end	
		FuBarPlugin:OpenTooltip(frame, addon)
	end
	
	brokerPlugin.OnLeave = function(frame)
		if addon.blizzardTooltip then
			GameTooltip:Hide()
		end	
	end
end

function FuBarPlugin:OnInstanceInit(target)
	local folderName
	for i = 6, 3, -1 do
		folderName = debugstack(i, 1, 0):match("\\AddOns\\(.*)\\")
		if folderName then
			break
		end
	end
	target.folderName = folderName
	folderNames[target] = folderName
end

function FuBarPlugin:GetAceOptionsDataTable()
	return {}
end

function FuBarPlugin:SetIcon(path)
	if brokerPlugins[self] then
		if type(path) ~= "string" then
			path = format("Interface\\AddOns\\%s\\icon", folderNames[self] or self.folderName)
		elseif not path:find('^Interface[\\/]') then
			path = format("Interface\\AddOns\\%s\\%s", folderNames[self] or self.folderName, path)
		end		
		brokerPlugins[self].icon = path
	end
end
function FuBarPlugin:SetText(text)
	if brokerPlugins[self] then
		brokerPlugins[self].text = text
	end
end

function FuBarPlugin:UpdateDisplay()
	self:UpdateText()
end

function FuBarPlugin:GetTitle()
	local name = self.title or self.name
	return name
end

function FuBarPlugin:UpdateText()
	if type(self.OnTextUpdate) == "function" then
		self:OnTextUpdate()
	else
		self:SetText(self:GetTitle())
	end
end

function FuBarPlugin:OpenMenu(frame, addon)
	local Dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
	
	if not Dewdrop:IsRegistered(frame) then
		if type(addon.OnMenuRequest) == "table" and (not addon.OnMenuRequest.handler or addon.OnMenuRequest.handler == frame) and addon.OnMenuRequest.type == "group" then
			Dewdrop:InjectAceOptionsTable(frame, addon.OnMenuRequest)
		end
		Dewdrop:Register(frame,
			'children', type(addon.OnMenuRequest) == "table" and addon.OnMenuRequest or function(level, value, valueN_1, valueN_2, valueN_3, valueN_4)
				if level == 1 then
					Dewdrop:AddLine(
						'text', addon:GetTitle(),
						'isTitle', true
					)
					if addon.OnMenuRequest then
						addon:OnMenuRequest(level, value, false, valueN_1, valueN_2, valueN_3, valueN_4)
					end
				end
			end,
			'point', function(frame)
				local x, y = frame:GetCenter()
				local leftRight
				if x < GetScreenWidth() / 2 then
					leftRight = "LEFT"
				else
					leftRight = "RIGHT"
				end
				if y < GetScreenHeight() / 2 then
					return "BOTTOM" .. leftRight, "TOP" .. leftRight
				else
					return "TOP" .. leftRight, "BOTTOM" .. leftRight
				end
			end,
			'dontHook', true
		)
	end
	Dewdrop:Open(frame)
end

function FuBarPlugin:TabletRegister(frame, addon)
	local tablet = AceLibrary:HasInstance("Tablet-2.0") and AceLibrary("Tablet-2.0")
	tablet:Register(frame,
		'children', function()
			tablet:SetTitle(addon:GetTitle())
			if type(addon.OnTooltipUpdate) == "function" then
				addon:OnTooltipUpdate()
			end
		end,
		'point', function(frame)
			if frame:GetTop() > GetScreenHeight() / 2 then
				local x = frame:GetCenter()
				if x < GetScreenWidth() / 3 then
					return "TOPLEFT", "BOTTOMLEFT"
				elseif x < GetScreenWidth() * 2 / 3 then
					return "TOP", "BOTTOM"
				else
					return "TOPRIGHT", "BOTTOMRIGHT"
				end
			else
				local x = frame:GetCenter()
				if x < GetScreenWidth() / 3 then
					return "BOTTOMLEFT", "TOPLEFT"
				elseif x < GetScreenWidth() * 2 / 3 then
					return "BOTTOM", "TOP"
				else
					return "BOTTOMRIGHT", "TOPRIGHT"
				end
			end
		end,
		'clickable', addon.clickableTooltip,
		'hideWhenEmpty', addon.tooltipHiddenWhenEmpty
	)	
end

function FuBarPlugin:OpenTooltip(frame, addon)
	local tablet = AceLibrary:HasInstance("Tablet-2.0") and AceLibrary("Tablet-2.0")
	if addon.blizzardTooltip then
		local anchor
		if frame:GetTop() > GetScreenHeight() / 2 then
			local x = frame:GetCenter()
			if x < GetScreenWidth() / 2 then
				anchor = "ANCHOR_BOTTOMRIGHT"
			else
				anchor = "ANCHOR_BOTTOMLEFT"
			end
		else
			local x = frame:GetCenter()
			if x < GetScreenWidth() / 2 then
				anchor = "ANCHOR_TOPLEFT"
			else
				anchor = "ANCHOR_TOPRIGHT"
			end
		end
		GameTooltip:SetOwner(frame, anchor)
		if type(addon.OnTooltipUpdate) == "function" then
			addon:OnTooltipUpdate()
		end
		GameTooltip:Show()			
	else
		tablet:Refresh(frame)
	end
end

local function activate(self, oldLib, oldDeactivate)
	FuBarPlugin = self

	FuBarPlugin.activate(self, oldLib, oldDeactivate)

	if oldDeactivate then
		oldDeactivate(oldLib)
	end
end

AceLibrary:Register(FuBarPlugin, MAJOR_VERSION, MINOR_VERSION, activate)